@@ -19,6 +19,7 @@ from http import client as http_client
 from unittest import mock
 
 import fixtures
+from keystonemiddleware import auth_token
 from oslo_config import cfg
 from oslo_utils import uuidutils
 
@@ -27,6 +28,7 @@ from ironic.api.controllers import v1 as api_v1
 from ironic.api.controllers.v1 import ramdisk
 from ironic.common import states
 from ironic.conductor import rpcapi
+from ironic.drivers.modules import inspect_utils
 from ironic.tests.unit.api import base as test_api_base
 from ironic.tests.unit.objects import utils as obj_utils
 
@@ -78,6 +80,9 @@ class TestLookup(test_api_base.BaseApiTest):
             'heartbeat_timeout': CONF.api.ramdisk_heartbeat_timeout,
             'agent_token': mock.ANY,
             'agent_token_required': True,
+            'agent_md5_checksum_enable': CONF.agent.allow_md5_checksum,
+            'disable_deep_image_inspection': CONF.conductor.disable_deep_image_inspection,  # noqa
+            'permitted_image_formats': CONF.conductor.permitted_image_formats,
         }
         self.assertEqual(expected_config, data['config'])
         self.assertIsNotNone(data['config']['agent_token'])
@@ -225,7 +230,8 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
                                                node.uuid, 'url', None, 'x',
-                                               None, topic='test-topic')
+                                               None, None, None,
+                                               topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_ok_with_json(self, mock_heartbeat):
@@ -240,7 +246,8 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
                                                node.uuid, 'url', None,
                                                'maybe some magic',
-                                               None, topic='test-topic')
+                                               None, None, None,
+                                               topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_ok_by_name(self, mock_heartbeat):
@@ -254,8 +261,8 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
                                                node.uuid, 'url', None,
-                                               'token',
-                                               None, topic='test-topic')
+                                               'token', None, None, None,
+                                               topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_ok_agent_version(self, mock_heartbeat):
@@ -271,7 +278,8 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
                                                node.uuid, 'url', '1.4.1',
                                                'meow',
-                                               None, topic='test-topic')
+                                               None, None, None,
+                                               topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_old_API_agent_version_error(self, mock_heartbeat):
@@ -308,7 +316,7 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
                                                node.uuid, 'url', None,
-                                               'abcdef1', None,
+                                               'abcdef1', None, None, None,
                                                topic='test-topic')
 
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
@@ -324,9 +332,41 @@ class TestHeartbeat(test_api_base.BaseApiTest):
         self.assertEqual(b'', response.body)
         mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
                                                node.uuid, 'url', None,
-                                               'meow', 'abcdef1',
+                                               'meow', 'abcdef1', None, None,
                                                topic='test-topic')
 
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_ok_agent_status_and_status(self, mock_heartbeat):
+        node = obj_utils.create_test_node(self.context)
+        response = self.post_json(
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': 'url',
+             'agent_token': 'meow',
+             'agent_status': 'start',
+             'agent_status_message': 'woof',
+             'agent_verify_ca': 'abcdef1'},
+            headers={api_base.Version.string: str(api_v1.max_version())})
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual(b'', response.body)
+        mock_heartbeat.assert_called_once_with(mock.ANY, mock.ANY,
+                                               node.uuid, 'url', None,
+                                               'meow', 'abcdef1', 'start',
+                                               'woof', topic='test-topic')
+
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_bad_invalid_agent_status(self, mock_heartbeat):
+        node = obj_utils.create_test_node(self.context)
+        response = self.post_json(
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': 'url',
+             'agent_token': 'meow',
+             'agent_status': 'invalid_state',
+             'agent_status_message': 'woof',
+             'agent_verify_ca': 'abcdef1'},
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
     @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
     def test_old_API_agent_verify_ca_error(self, mock_heartbeat):
         node = obj_utils.create_test_node(self.context)
@@ -338,3 +378,165 @@ class TestHeartbeat(test_api_base.BaseApiTest):
             headers={api_base.Version.string: '1.67'},
             expect_errors=True)
         self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
+    @mock.patch.object(rpcapi.ConductorAPI, 'heartbeat', autospec=True)
+    def test_old_api_agent_status_error(self, mock_heartbeat):
+        node = obj_utils.create_test_node(self.context)
+        response = self.post_json(
+            '/heartbeat/%s' % node.uuid,
+            {'callback_url': 'url',
+             'agent_token': 'meow',
+             'agent_verify_ca': 'abcd',
+             'agent_status': 'wow',
+             'agent_status_message': 'much status'},
+            headers={api_base.Version.string: '1.71'},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+
+
+@mock.patch.object(auth_token.AuthProtocol, 'process_request',
+                   lambda *_: None)
+class TestLookupScopedRBAC(TestLookup):
+
+    """Test class to execute the Lookup tests with RBAC enforcement."""
+    def setUp(self):
+        super(TestLookupScopedRBAC, self).setUp()
+
+        cfg.CONF.set_override('enforce_scope', True, group='oslo_policy')
+        cfg.CONF.set_override('enforce_new_defaults', True,
+                              group='oslo_policy')
+        cfg.CONF.set_override('auth_strategy', 'keystone')
+
+
+@mock.patch.object(auth_token.AuthProtocol, 'process_request',
+                   lambda *_: None)
+class TestHeartbeatScopedRBAC(TestHeartbeat):
+
+    """Test class to execute the Heartbeat tests with RBAC enforcement."""
+    def setUp(self):
+        super(TestHeartbeatScopedRBAC, self).setUp()
+
+        cfg.CONF.set_override('enforce_scope', True, group='oslo_policy')
+        cfg.CONF.set_override('enforce_new_defaults', True,
+                              group='oslo_policy')
+        cfg.CONF.set_override('auth_strategy', 'keystone')
+
+
+@mock.patch.object(rpcapi.ConductorAPI, 'get_topic_for',
+                   lambda *n: 'test-topic')
+@mock.patch.object(rpcapi.ConductorAPI, 'continue_inspection', autospec=True)
+@mock.patch.object(inspect_utils, 'lookup_node', autospec=True)
+class TestContinueInspection(test_api_base.BaseApiTest):
+
+    def setUp(self):
+        super().setUp()
+        self.addresses = ['11:22:33:44:55:66', '66:55:44:33:22:11']
+        self.bmcs = ['192.0.2.42', '2001:db8::42']
+        self.inventory = {
+            'bmc_address': self.bmcs[0],
+            'bmc_v6address': self.bmcs[1],
+            'interfaces': [
+                {'mac_address': mac, 'name': f'em{i}'}
+                for i, mac in enumerate(self.addresses)
+            ],
+        }
+        self.data = {
+            'inventory': self.inventory,
+            'test': 42,
+        }
+        self.node = obj_utils.create_test_node(self.context,
+                                               uuid=uuidutils.generate_uuid(),
+                                               provision_state='inspect wait')
+
+    def test_inspector_compatibility(self, mock_lookup, mock_continue):
+        mock_lookup.return_value = self.node
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual({'uuid': self.node.uuid}, response.json)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42}, topic='test-topic')
+
+    def test_bmc_address_as_none(self, mock_lookup, mock_continue):
+        mock_lookup.return_value = self.node
+        self.inventory['bmc_address'] = None
+        self.inventory['bmc_v6address'] = None
+        response = self.post_json('/continue_inspection', self.data)
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual({'uuid': self.node.uuid}, response.json)
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, [], node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42}, topic='test-topic')
+
+    @mock.patch.object(rpcapi.ConductorAPI, 'get_node_with_token',
+                       autospec=True)
+    def test_new_api(self, mock_get_node, mock_lookup, mock_continue):
+        mock_lookup.return_value = self.node
+        mock_get_node.return_value = self.node
+        response = self.post_json(
+            '/continue_inspection', self.data,
+            headers={api_base.Version.string: str(api_v1.max_version())})
+        self.assertEqual(http_client.ACCEPTED, response.status_int)
+        self.assertEqual(self.node.uuid, response.json['node']['uuid'])
+        self.assertEqual(set(ramdisk._LOOKUP_RETURN_FIELDS) | {'links'},
+                         set(response.json['node']))
+        mock_lookup.assert_called_once_with(
+            mock.ANY, self.addresses, self.bmcs, node_uuid=None)
+        mock_continue.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, inventory=self.inventory,
+            plugin_data={'test': 42}, topic='test-topic')
+        mock_get_node.assert_called_once_with(
+            mock.ANY, mock.ANY, self.node.uuid, topic='test-topic')
+
+    def test_old_api_version(self, mock_lookup, mock_continue):
+        response = self.post_json(
+            '/continue_inspection', self.data,
+            headers={api_base.Version.string: '1.83'},
+            expect_errors=True)
+        self.assertEqual(http_client.NOT_FOUND, response.status_int)
+        self.assertIn("API version", response.text)
+        mock_lookup.assert_not_called()
+        mock_continue.assert_not_called()
+
+    def test_invalid_schema(self, mock_lookup, mock_continue):
+        del self.data['inventory']['interfaces']
+        response = self.post_json(
+            '/continue_inspection', self.data,
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        # JSON schema errors can change between versions, so only make sure
+        # it mentions the required field.
+        self.assertIn("interfaces", response.text)
+        mock_lookup.assert_not_called()
+        mock_continue.assert_not_called()
+
+    def test_no_usable_lookup_data(self, mock_lookup, mock_continue):
+        self.data['inventory']['interfaces'] = [{'mac_address': 'meow'}]
+        del self.data['inventory']['bmc_address']
+        del self.data['inventory']['bmc_v6address']
+        response = self.post_json(
+            '/continue_inspection', self.data,
+            headers={api_base.Version.string: str(api_v1.max_version())},
+            expect_errors=True)
+        self.assertEqual(http_client.BAD_REQUEST, response.status_int)
+        self.assertIn("No lookup information", response.text)
+        mock_lookup.assert_not_called()
+        mock_continue.assert_not_called()
+
+
+@mock.patch.object(auth_token.AuthProtocol, 'process_request',
+                   lambda *_: None)
+class TestContinueInspectionScopedRBAC(TestContinueInspection):
+
+    def setUp(self):
+        super().setUp()
+
+        cfg.CONF.set_override('enforce_scope', True, group='oslo_policy')
+        cfg.CONF.set_override('enforce_new_defaults', True,
+                              group='oslo_policy')
+        cfg.CONF.set_override('auth_strategy', 'keystone')
