@@ -23,9 +23,11 @@ import time
 from unittest import mock
 import uuid
 
+from oslo_config import cfg
 from oslo_utils import uuidutils
 
 from ironic.common import exception
+from ironic.common import image_format_inspector
 from ironic.common import image_service
 from ironic.common import images
 from ironic.common import utils
@@ -37,10 +39,10 @@ def touch(filename):
     open(filename, 'w').close()
 
 
-class TestImageCacheFetch(base.TestCase):
+class BaseTest(base.TestCase):
 
     def setUp(self):
-        super(TestImageCacheFetch, self).setUp()
+        super().setUp()
         self.master_dir = tempfile.mkdtemp()
         self.cache = image_cache.ImageCache(self.master_dir, None, None)
         self.dest_dir = tempfile.mkdtemp()
@@ -48,23 +50,44 @@ class TestImageCacheFetch(base.TestCase):
         self.uuid = uuidutils.generate_uuid()
         self.master_path = ''.join([os.path.join(self.master_dir, self.uuid),
                                     '.converted'])
+        self.img_info = {}
+
+
+@mock.patch.object(image_service, 'get_image_service', autospec=True)
+@mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
+@mock.patch.object(image_cache.ImageCache, '_download_image', autospec=True)
+class TestImageCacheFetch(BaseTest):
 
     @mock.patch.object(image_cache, '_fetch', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
-    def test_fetch_image_no_master_dir(self, mock_download, mock_clean_up,
-                                       mock_fetch):
+    def test_fetch_image_no_master_dir(self, mock_fetch, mock_download,
+                                       mock_clean_up, mock_image_service):
         self.cache.master_dir = None
         self.cache.fetch_image(self.uuid, self.dest_path)
         self.assertFalse(mock_download.called)
         mock_fetch.assert_called_once_with(
-            None, self.uuid, self.dest_path, True)
+            None, self.uuid, self.dest_path, True,
+            None, None, None)
         self.assertFalse(mock_clean_up.called)
+        mock_image_service.assert_not_called()
+
+    @mock.patch.object(image_cache, '_fetch', autospec=True)
+    def test_fetch_image_no_master_dir_memory_low(self,
+                                                  mock_fetch,
+                                                  mock_download,
+                                                  mock_clean_up,
+                                                  mock_image_service):
+        mock_fetch.side_effect = exception.InsufficientMemory
+        self.cache.master_dir = None
+        self.assertRaises(exception.InsufficientMemory,
+                          self.cache.fetch_image,
+                          self.uuid, self.dest_path)
+        self.assertFalse(mock_download.called)
+        mock_fetch.assert_called_once_with(
+            None, self.uuid, self.dest_path, True,
+            None, None, None)
+        self.assertFalse(mock_clean_up.called)
+        mock_image_service.assert_not_called()
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(image_cache, '_delete_dest_path_if_stale',
                        return_value=True, autospec=True)
@@ -72,18 +95,18 @@ class TestImageCacheFetch(base.TestCase):
                        return_value=True, autospec=True)
     def test_fetch_image_dest_and_master_uptodate(
             self, mock_cache_upd, mock_dest_upd, mock_link, mock_download,
-            mock_clean_up):
+            mock_clean_up, mock_image_service):
         self.cache.fetch_image(self.uuid, self.dest_path)
-        mock_cache_upd.assert_called_once_with(self.master_path, self.uuid,
-                                               None)
+        mock_cache_upd.assert_called_once_with(
+            self.master_path, self.uuid,
+            mock_image_service.return_value.show.return_value)
         mock_dest_upd.assert_called_once_with(self.master_path, self.dest_path)
         self.assertFalse(mock_link.called)
         self.assertFalse(mock_download.called)
         self.assertFalse(mock_clean_up.called)
+        mock_image_service.assert_called_once_with(self.uuid, context=None)
+        mock_image_service.return_value.show.assert_called_once_with(self.uuid)
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(image_cache, '_delete_dest_path_if_stale',
                        return_value=True, autospec=True)
@@ -91,19 +114,17 @@ class TestImageCacheFetch(base.TestCase):
                        return_value=True, autospec=True)
     def test_fetch_image_dest_and_master_uptodate_no_force_raw(
             self, mock_cache_upd, mock_dest_upd, mock_link, mock_download,
-            mock_clean_up):
+            mock_clean_up, mock_image_service):
         master_path = os.path.join(self.master_dir, self.uuid)
         self.cache.fetch_image(self.uuid, self.dest_path, force_raw=False)
-        mock_cache_upd.assert_called_once_with(master_path, self.uuid,
-                                               None)
+        mock_cache_upd.assert_called_once_with(
+            master_path, self.uuid,
+            mock_image_service.return_value.show.return_value)
         mock_dest_upd.assert_called_once_with(master_path, self.dest_path)
         self.assertFalse(mock_link.called)
         self.assertFalse(mock_download.called)
         self.assertFalse(mock_clean_up.called)
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(image_cache, '_delete_dest_path_if_stale',
                        return_value=False, autospec=True)
@@ -111,18 +132,16 @@ class TestImageCacheFetch(base.TestCase):
                        return_value=True, autospec=True)
     def test_fetch_image_dest_out_of_date(
             self, mock_cache_upd, mock_dest_upd, mock_link, mock_download,
-            mock_clean_up):
+            mock_clean_up, mock_image_service):
         self.cache.fetch_image(self.uuid, self.dest_path)
-        mock_cache_upd.assert_called_once_with(self.master_path, self.uuid,
-                                               None)
+        mock_cache_upd.assert_called_once_with(
+            self.master_path, self.uuid,
+            mock_image_service.return_value.show.return_value)
         mock_dest_upd.assert_called_once_with(self.master_path, self.dest_path)
         mock_link.assert_called_once_with(self.master_path, self.dest_path)
         self.assertFalse(mock_download.called)
         self.assertFalse(mock_clean_up.called)
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(image_cache, '_delete_dest_path_if_stale',
                        return_value=True, autospec=True)
@@ -130,20 +149,22 @@ class TestImageCacheFetch(base.TestCase):
                        return_value=False, autospec=True)
     def test_fetch_image_master_out_of_date(
             self, mock_cache_upd, mock_dest_upd, mock_link, mock_download,
-            mock_clean_up):
+            mock_clean_up, mock_image_service):
         self.cache.fetch_image(self.uuid, self.dest_path)
-        mock_cache_upd.assert_called_once_with(self.master_path, self.uuid,
-                                               None)
+        mock_cache_upd.assert_called_once_with(
+            self.master_path, self.uuid,
+            mock_image_service.return_value.show.return_value)
         mock_dest_upd.assert_called_once_with(self.master_path, self.dest_path)
         self.assertFalse(mock_link.called)
         mock_download.assert_called_once_with(
             self.cache, self.uuid, self.master_path, self.dest_path,
-            ctx=None, force_raw=True)
+            mock_image_service.return_value.show.return_value,
+            ctx=None, force_raw=True, expected_format=None,
+            expected_checksum=None, expected_checksum_algo=None)
         mock_clean_up.assert_called_once_with(self.cache)
+        mock_image_service.assert_called_once_with(self.uuid, context=None)
+        mock_image_service.return_value.show.assert_called_once_with(self.uuid)
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(image_cache, '_delete_dest_path_if_stale',
                        return_value=True, autospec=True)
@@ -151,21 +172,22 @@ class TestImageCacheFetch(base.TestCase):
                        return_value=False, autospec=True)
     def test_fetch_image_both_master_and_dest_out_of_date(
             self, mock_cache_upd, mock_dest_upd, mock_link, mock_download,
-            mock_clean_up):
+            mock_clean_up, mock_image_service):
         self.cache.fetch_image(self.uuid, self.dest_path)
-        mock_cache_upd.assert_called_once_with(self.master_path, self.uuid,
-                                               None)
+        mock_cache_upd.assert_called_once_with(
+            self.master_path, self.uuid,
+            mock_image_service.return_value.show.return_value)
         mock_dest_upd.assert_called_once_with(self.master_path, self.dest_path)
         self.assertFalse(mock_link.called)
         mock_download.assert_called_once_with(
             self.cache, self.uuid, self.master_path, self.dest_path,
-            ctx=None, force_raw=True)
+            mock_image_service.return_value.show.return_value,
+            ctx=None, force_raw=True, expected_format=None,
+            expected_checksum=None, expected_checksum_algo=None)
         mock_clean_up.assert_called_once_with(self.cache)
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
-    def test_fetch_image_not_uuid(self, mock_download, mock_clean_up):
+    def test_fetch_image_not_uuid(self, mock_download, mock_clean_up,
+                                  mock_image_service):
         href = u'http://abc.com/ubuntu.qcow2'
         href_converted = str(uuid.uuid5(uuid.NAMESPACE_URL, href))
         master_path = ''.join([os.path.join(self.master_dir, href_converted),
@@ -173,26 +195,34 @@ class TestImageCacheFetch(base.TestCase):
         self.cache.fetch_image(href, self.dest_path)
         mock_download.assert_called_once_with(
             self.cache, href, master_path, self.dest_path,
-            ctx=None, force_raw=True)
+            mock_image_service.return_value.show.return_value,
+            ctx=None, force_raw=True, expected_format=None,
+            expected_checksum=None, expected_checksum_algo=None)
         self.assertTrue(mock_clean_up.called)
 
-    @mock.patch.object(image_cache.ImageCache, 'clean_up', autospec=True)
-    @mock.patch.object(image_cache.ImageCache, '_download_image',
-                       autospec=True)
     def test_fetch_image_not_uuid_no_force_raw(self, mock_download,
-                                               mock_clean_up):
+                                               mock_clean_up,
+                                               mock_image_service):
         href = u'http://abc.com/ubuntu.qcow2'
         href_converted = str(uuid.uuid5(uuid.NAMESPACE_URL, href))
         master_path = os.path.join(self.master_dir, href_converted)
-        self.cache.fetch_image(href, self.dest_path, force_raw=False)
+        self.cache.fetch_image(href, self.dest_path, force_raw=False,
+                               expected_checksum='f00',
+                               expected_checksum_algo='sha256')
         mock_download.assert_called_once_with(
             self.cache, href, master_path, self.dest_path,
-            ctx=None, force_raw=False)
+            mock_image_service.return_value.show.return_value,
+            ctx=None, force_raw=False, expected_format=None,
+            expected_checksum='f00', expected_checksum_algo='sha256')
         self.assertTrue(mock_clean_up.called)
 
-    @mock.patch.object(image_cache, '_fetch', autospec=True)
+
+@mock.patch.object(image_cache, '_fetch', autospec=True)
+class TestImageCacheDownload(BaseTest):
+
     def test__download_image(self, mock_fetch):
-        def _fake_fetch(ctx, uuid, tmp_path, *args):
+        def _fake_fetch(ctx, uuid, tmp_path, force_raw, expected_format,
+                        expected_checksum, expected_checksum_algo):
             self.assertEqual(self.uuid, uuid)
             self.assertNotEqual(self.dest_path, tmp_path)
             self.assertNotEqual(os.path.dirname(tmp_path), self.master_dir)
@@ -200,7 +230,31 @@ class TestImageCacheFetch(base.TestCase):
                 fp.write("TEST")
 
         mock_fetch.side_effect = _fake_fetch
-        self.cache._download_image(self.uuid, self.master_path, self.dest_path)
+        self.cache._download_image(self.uuid, self.master_path, self.dest_path,
+                                   self.img_info)
+        self.assertTrue(os.path.isfile(self.dest_path))
+        self.assertTrue(os.path.isfile(self.master_path))
+        self.assertEqual(os.stat(self.dest_path).st_ino,
+                         os.stat(self.master_path).st_ino)
+        with open(self.dest_path) as fp:
+            self.assertEqual("TEST", fp.read())
+
+    def test__download_image_large_url(self, mock_fetch):
+        # A long enough URL may exceed the file name limits of the file system.
+        # Make sure we don't use any parts of the URL anywhere.
+        url = "http://example.com/image.iso?secret=%s" % ("x" * 1000)
+
+        def _fake_fetch(ctx, href, tmp_path, force_raw, expected_format,
+                        expected_checksum, expected_checksum_algo):
+            self.assertEqual(url, href)
+            self.assertNotEqual(self.dest_path, tmp_path)
+            self.assertNotEqual(os.path.dirname(tmp_path), self.master_dir)
+            with open(tmp_path, 'w') as fp:
+                fp.write("TEST")
+
+        mock_fetch.side_effect = _fake_fetch
+        self.cache._download_image(url, self.master_path, self.dest_path,
+                                   self.img_info)
         self.assertTrue(os.path.isfile(self.dest_path))
         self.assertTrue(os.path.isfile(self.master_path))
         self.assertEqual(os.stat(self.dest_path).st_ino,
@@ -208,115 +262,97 @@ class TestImageCacheFetch(base.TestCase):
         with open(self.dest_path) as fp:
             self.assertEqual("TEST", fp.read())
 
-    @mock.patch.object(image_cache, '_fetch', autospec=True)
     @mock.patch.object(image_cache, 'LOG', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     def test__download_image_linkfail(self, mock_link, mock_log, mock_fetch):
         mock_link.side_effect = [None, OSError]
         self.assertRaises(exception.ImageDownloadFailed,
                           self.cache._download_image,
-                          self.uuid, self.master_path, self.dest_path)
+                          self.uuid, self.master_path, self.dest_path,
+                          self.img_info)
         self.assertTrue(mock_fetch.called)
         self.assertEqual(2, mock_link.call_count)
         self.assertTrue(mock_log.error.called)
 
+    def test__download_image_raises_memory_guard(self, mock_fetch):
+        mock_fetch.side_effect = exception.InsufficientMemory
+        self.assertRaises(exception.InsufficientMemory,
+                          self.cache._download_image,
+                          self.uuid, self.master_path,
+                          self.dest_path, self.img_info)
+
 
 @mock.patch.object(os, 'unlink', autospec=True)
-class TestUpdateImages(base.TestCase):
-
-    def setUp(self):
-        super(TestUpdateImages, self).setUp()
-        self.master_dir = tempfile.mkdtemp()
-        self.dest_dir = tempfile.mkdtemp()
-        self.dest_path = os.path.join(self.dest_dir, 'dest')
-        self.uuid = uuidutils.generate_uuid()
-        self.master_path = os.path.join(self.master_dir, self.uuid)
+class TestUpdateImages(BaseTest):
 
     @mock.patch.object(os.path, 'exists', return_value=False, autospec=True)
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
     def test__delete_master_path_if_stale_glance_img_not_cached(
-            self, mock_gis, mock_path_exists, mock_unlink):
+            self, mock_path_exists, mock_unlink):
         res = image_cache._delete_master_path_if_stale(self.master_path,
-                                                       self.uuid, None)
-        self.assertFalse(mock_gis.called)
+                                                       self.uuid,
+                                                       self.img_info)
         self.assertFalse(mock_unlink.called)
         mock_path_exists.assert_called_once_with(self.master_path)
         self.assertFalse(res)
 
     @mock.patch.object(os.path, 'exists', return_value=True, autospec=True)
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
     def test__delete_master_path_if_stale_glance_img(
-            self, mock_gis, mock_path_exists, mock_unlink):
+            self, mock_path_exists, mock_unlink):
         res = image_cache._delete_master_path_if_stale(self.master_path,
-                                                       self.uuid, None)
-        self.assertFalse(mock_gis.called)
+                                                       self.uuid,
+                                                       self.img_info)
         self.assertFalse(mock_unlink.called)
         mock_path_exists.assert_called_once_with(self.master_path)
         self.assertTrue(res)
 
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
-    def test__delete_master_path_if_stale_no_master(self, mock_gis,
-                                                    mock_unlink):
+    def test__delete_master_path_if_stale_no_master(self, mock_unlink):
         res = image_cache._delete_master_path_if_stale(self.master_path,
-                                                       'http://11', None)
-        self.assertFalse(mock_gis.called)
+                                                       'http://11',
+                                                       self.img_info)
         self.assertFalse(mock_unlink.called)
         self.assertFalse(res)
 
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
-    def test__delete_master_path_if_stale_no_updated_at(self, mock_gis,
-                                                        mock_unlink):
+    def test__delete_master_path_if_stale_no_updated_at(self, mock_unlink):
         touch(self.master_path)
         href = 'http://awesomefreeimages.al/img111'
-        mock_gis.return_value.show.return_value = {}
         res = image_cache._delete_master_path_if_stale(self.master_path, href,
-                                                       None)
-        mock_gis.assert_called_once_with(href, context=None)
-        self.assertFalse(mock_unlink.called)
-        self.assertTrue(res)
+                                                       self.img_info)
+        mock_unlink.assert_called_once_with(self.master_path)
+        self.assertFalse(res)
 
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
-    def test__delete_master_path_if_stale_master_up_to_date(self, mock_gis,
-                                                            mock_unlink):
+    def test__delete_master_path_if_stale_master_up_to_date(self, mock_unlink):
         touch(self.master_path)
         href = 'http://awesomefreeimages.al/img999'
-        mock_gis.return_value.show.return_value = {
+        self.img_info = {
             'updated_at': datetime.datetime(1999, 11, 15, 8, 12, 31)
         }
         res = image_cache._delete_master_path_if_stale(self.master_path, href,
-                                                       None)
-        mock_gis.assert_called_once_with(href, context=None)
+                                                       self.img_info)
         self.assertFalse(mock_unlink.called)
         self.assertTrue(res)
 
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
-    def test__delete_master_path_if_stale_master_same_time(self, mock_gis,
-                                                           mock_unlink):
+    def test__delete_master_path_if_stale_master_same_time(self, mock_unlink):
         # When times identical should not delete cached file
         touch(self.master_path)
         mtime = utils.unix_file_modification_datetime(self.master_path)
         href = 'http://awesomefreeimages.al/img999'
-        mock_gis.return_value.show.return_value = {
+        self.img_info = {
             'updated_at': mtime
         }
         res = image_cache._delete_master_path_if_stale(self.master_path, href,
-                                                       None)
-        mock_gis.assert_called_once_with(href, context=None)
+                                                       self.img_info)
         self.assertFalse(mock_unlink.called)
         self.assertTrue(res)
 
-    @mock.patch.object(image_service, 'get_image_service', autospec=True)
-    def test__delete_master_path_if_stale_out_of_date(self, mock_gis,
-                                                      mock_unlink):
+    def test__delete_master_path_if_stale_out_of_date(self, mock_unlink):
         touch(self.master_path)
         href = 'http://awesomefreeimages.al/img999'
-        mock_gis.return_value.show.return_value = {
+        self.img_info = {
             'updated_at': datetime.datetime((datetime.datetime.utcnow().year
                                              + 1), 11, 15, 8, 12, 31)
         }
         res = image_cache._delete_master_path_if_stale(self.master_path, href,
-                                                       None)
-        mock_gis.assert_called_once_with(href, context=None)
+                                                       self.img_info)
         mock_unlink.assert_called_once_with(self.master_path)
         self.assertFalse(res)
 
@@ -493,7 +529,8 @@ class TestImageCacheCleanUp(base.TestCase):
     @mock.patch.object(utils, 'rmtree_without_raise', autospec=True)
     @mock.patch.object(image_cache, '_fetch', autospec=True)
     def test_temp_images_not_cleaned(self, mock_fetch, mock_rmtree):
-        def _fake_fetch(ctx, uuid, tmp_path, *args):
+        def _fake_fetch(ctx, uuid, tmp_path, force_raw, expected_format,
+                        expected_checksum, expected_checksum_algo):
             with open(tmp_path, 'w') as fp:
                 fp.write("TEST" * 10)
 
@@ -504,7 +541,7 @@ class TestImageCacheCleanUp(base.TestCase):
         mock_fetch.side_effect = _fake_fetch
         master_path = os.path.join(self.master_dir, 'uuid')
         dest_path = os.path.join(tempfile.mkdtemp(), 'dest')
-        self.cache._download_image('uuid', master_path, dest_path)
+        self.cache._download_image('uuid', master_path, dest_path, {})
         self.assertTrue(mock_rmtree.called)
 
     @mock.patch.object(utils, 'rmtree_without_raise', autospec=True)
@@ -513,7 +550,7 @@ class TestImageCacheCleanUp(base.TestCase):
         mock_fetch.side_effect = exception.IronicException
         self.assertRaises(exception.IronicException,
                           self.cache._download_image,
-                          'uuid', 'fake', 'fake')
+                          'uuid', 'fake', 'fake', {})
         self.assertTrue(mock_rmtree.called)
 
     @mock.patch.object(image_cache.LOG, 'warning', autospec=True)
@@ -730,38 +767,266 @@ class CleanupImageCacheTestCase(base.TestCase):
 
 class TestFetchCleanup(base.TestCase):
 
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
     @mock.patch.object(images, 'converted_size', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
     @mock.patch.object(images, 'image_to_raw', autospec=True)
-    @mock.patch.object(images, 'force_raw_will_convert', autospec=True,
-                       return_value=True)
     @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
     def test__fetch(
-            self, mock_clean, mock_will_convert, mock_raw, mock_fetch,
-            mock_size):
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_remove, mock_show, mock_format_inspector):
+        image_check = mock.MagicMock()
+        image_check.__str__.side_effect = iter(['qcow2', 'raw'])
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        mock_show.return_value = {}
         mock_size.return_value = 100
-        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True,
+                           expected_checksum='1234',
+                           expected_checksum_algo='md5')
         mock_fetch.assert_called_once_with('fake', 'fake-uuid',
-                                           '/foo/bar.part', force_raw=False)
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum='1234',
+                                           checksum_algo='md5')
         mock_clean.assert_called_once_with('/foo', 100)
         mock_raw.assert_called_once_with('fake-uuid', '/foo/bar',
                                          '/foo/bar.part')
-        mock_will_convert.assert_called_once_with('fake-uuid', '/foo/bar.part')
+        mock_remove.assert_not_called()
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(1, image_check.__str__.call_count)
 
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
     @mock.patch.object(images, 'converted_size', autospec=True)
     @mock.patch.object(images, 'fetch', autospec=True)
     @mock.patch.object(images, 'image_to_raw', autospec=True)
-    @mock.patch.object(images, 'force_raw_will_convert', autospec=True,
-                       return_value=False)
     @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
-    def test__fetch_already_raw(
-            self, mock_clean, mock_will_convert, mock_raw, mock_fetch,
-            mock_size):
+    def test__fetch_deep_inspection_disabled(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_remove, mock_show, mock_format_inspector):
+        cfg.CONF.set_override(
+            'disable_deep_image_inspection', True,
+            group='conductor')
+        image_check = mock.MagicMock()
+        image_check.__str__.return_value = 'qcow2'
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        mock_show.return_value = {}
+        mock_size.return_value = 100
         image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
         mock_fetch.assert_called_once_with('fake', 'fake-uuid',
-                                           '/foo/bar.part', force_raw=False)
-        mock_clean.assert_not_called()
-        mock_size.assert_not_called()
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum=None, checksum_algo=None)
+        mock_clean.assert_called_once_with('/foo', 100)
         mock_raw.assert_called_once_with('fake-uuid', '/foo/bar',
                                          '/foo/bar.part')
-        mock_will_convert.assert_called_once_with('fake-uuid', '/foo/bar.part')
+        mock_remove.assert_not_called()
+        mock_show.assert_not_called()
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_not_called()
+        self.assertEqual(1, image_check.__str__.call_count)
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_already_raw(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_show, mock_format_inspector,
+            mock_rename):
+        mock_show.return_value = {'disk_format': 'raw'}
+        image_check = mock.MagicMock()
+        image_check.__str__.return_value = 'raw'
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True,
+                           expected_checksum='e00',
+                           expected_checksum_algo='sha256')
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum='e00',
+                                           checksum_algo='sha256')
+        mock_clean.assert_not_called()
+        mock_size.assert_not_called()
+        mock_raw.assert_not_called()
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(1, image_check.__str__.call_count)
+        mock_rename.assert_called_once_with('/foo/bar.part', '/foo/bar')
+
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_format_does_not_match_glance(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_show, mock_format_inspector):
+        mock_show.return_value = {'disk_format': 'raw'}
+        image_check = mock.MagicMock()
+        image_check.__str__.return_value = 'qcow2'
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        self.assertRaises(exception.InvalidImage,
+                          image_cache._fetch,
+                          'fake', 'fake-uuid',
+                          '/foo/bar', force_raw=True,
+                          expected_format=None,
+                          expected_checksum='a00',
+                          expected_checksum_algo='sha512')
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum='a00',
+                                           checksum_algo='sha512')
+        mock_clean.assert_not_called()
+        mock_size.assert_not_called()
+        mock_raw.assert_not_called()
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(1, image_check.__str__.call_count)
+
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_not_safe_image(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_show, mock_format_inspector):
+        mock_show.return_value = {'disk_format': 'qcow2'}
+        image_check = mock.MagicMock()
+        image_check.__str__.return_value = 'qcow2'
+        image_check.safety_check.return_value = False
+        mock_format_inspector.return_value = image_check
+        self.assertRaises(exception.InvalidImage,
+                          image_cache._fetch,
+                          'fake', 'fake-uuid',
+                          '/foo/bar', force_raw=True)
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum=None, checksum_algo=None)
+        mock_clean.assert_not_called()
+        mock_size.assert_not_called()
+        mock_raw.assert_not_called()
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(0, image_check.__str__.call_count)
+
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_estimate_fallback(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_show, mock_format_inspector):
+        mock_show.return_value = {'disk_format': 'qcow2'}
+        image_check = mock.MagicMock()
+        image_check.__str__.side_effect = iter(['qcow2', 'raw'])
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        mock_size.side_effect = [100, 10]
+        mock_clean.side_effect = [exception.InsufficientDiskSpace(), None]
+
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum=None, checksum_algo=None)
+        mock_size.assert_has_calls([
+            mock.call('/foo/bar.part', estimate=False),
+            mock.call('/foo/bar.part', estimate=True),
+        ])
+        mock_clean.assert_has_calls([
+            mock.call('/foo', 100),
+            mock.call('/foo', 10),
+        ])
+        mock_raw.assert_called_once_with('fake-uuid', '/foo/bar',
+                                         '/foo/bar.part')
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(1, image_check.__str__.call_count)
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_ramdisk_kernel(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_remove, mock_show, mock_format_inspector,
+            mock_rename):
+        image_check = mock.MagicMock()
+        image_check.__str__.return_value = 'raw'
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        mock_show.return_value = {'disk_format': 'aki'}
+        mock_size.return_value = 100
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum=None, checksum_algo=None)
+        mock_clean.assert_not_called()
+        mock_raw.assert_not_called()
+        mock_remove.assert_not_called()
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(1, image_check.__str__.call_count)
+        mock_rename.assert_called_once_with('/foo/bar.part', '/foo/bar')
+
+    @mock.patch.object(os, 'rename', autospec=True)
+    @mock.patch.object(image_format_inspector, 'detect_file_format',
+                       autospec=True)
+    @mock.patch.object(images, 'image_show', autospec=True)
+    @mock.patch.object(os, 'remove', autospec=True)
+    @mock.patch.object(images, 'converted_size', autospec=True)
+    @mock.patch.object(images, 'fetch', autospec=True)
+    @mock.patch.object(images, 'image_to_raw', autospec=True)
+    @mock.patch.object(image_cache, '_clean_up_caches', autospec=True)
+    def test__fetch_ramdisk_image(
+            self, mock_clean, mock_raw, mock_fetch,
+            mock_size, mock_remove, mock_show, mock_format_inspector,
+            mock_rename):
+        image_check = mock.MagicMock()
+        image_check.__str__.return_value = 'raw'
+        image_check.safety_check.return_value = True
+        mock_format_inspector.return_value = image_check
+        mock_show.return_value = {'disk_format': 'ari'}
+        mock_size.return_value = 100
+        image_cache._fetch('fake', 'fake-uuid', '/foo/bar', force_raw=True)
+        mock_fetch.assert_called_once_with('fake', 'fake-uuid',
+                                           '/foo/bar.part', force_raw=False,
+                                           checksum=None, checksum_algo=None)
+        mock_clean.assert_not_called()
+        mock_raw.assert_not_called()
+        mock_remove.assert_not_called()
+        mock_show.assert_called_once_with('fake', 'fake-uuid')
+        mock_format_inspector.assert_called_once_with('/foo/bar.part')
+        image_check.safety_check.assert_called_once()
+        self.assertEqual(1, image_check.__str__.call_count)
+        mock_rename.assert_called_once_with('/foo/bar.part', '/foo/bar')
