@@ -105,73 +105,96 @@ class RedfishImageHandlerTestCase(db_base.DbTestCase):
             mock_swift_api.delete_object.assert_called_once_with(
                 'ironic_redfish_container', object_name)
 
+    @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_local_link(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
+            mock_execute):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost', group='deploy')
         img_handler_obj = image_utils.ImageHandler(self.node.driver)
-
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
-
         self.assertEqual(
             'http://localhost/redfish/boot.iso', url)
-
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
             'file.iso', '/httpboot/redfish/boot.iso')
         mock_chmod.assert_called_once_with('file.iso', 0o644)
+        mock_execute.assert_called_once_with(
+            '/usr/sbin/restorecon', '-i', '-R', 'v', '/httpboot/redfish')
 
+    @mock.patch.object(utils, 'execute', autospec=True)
+    @mock.patch.object(os, 'chmod', autospec=True)
+    @mock.patch.object(image_utils, 'shutil', autospec=True)
+    @mock.patch.object(os, 'link', autospec=True)
+    @mock.patch.object(os, 'mkdir', autospec=True)
+    def test_publish_image_local_link_no_restorecon(
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
+            mock_execute):
+        self.config(use_swift=False, group='redfish')
+        self.config(http_url='http://localhost', group='deploy')
+        img_handler_obj = image_utils.ImageHandler(self.node.driver)
+        url = img_handler_obj.publish_image('file.iso', 'boot.iso')
+        self.assertEqual(
+            'http://localhost/redfish/boot.iso', url)
+        mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
+        mock_link.assert_called_once_with(
+            'file.iso', '/httpboot/redfish/boot.iso')
+        mock_chmod.assert_called_once_with('file.iso', 0o644)
+        mock_execute.return_value = FileNotFoundError
+        mock_shutil.assert_not_called()
+
+    @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_external_ip(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
+            mock_execute):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost',
                     external_http_url='http://non-local.host',
                     group='deploy')
         img_handler_obj = image_utils.ImageHandler(self.node.driver)
-
         url = img_handler_obj.publish_image('file.iso', 'boot.iso')
-
         self.assertEqual(
             'http://non-local.host/redfish/boot.iso', url)
-
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
             'file.iso', '/httpboot/redfish/boot.iso')
         mock_chmod.assert_called_once_with('file.iso', 0o644)
+        mock_execute.assert_called_once_with(
+            '/usr/sbin/restorecon', '-i', '-R', 'v', '/httpboot/redfish')
 
+    @mock.patch.object(utils, 'execute', autospec=True)
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
     @mock.patch.object(os, 'link', autospec=True)
     @mock.patch.object(os, 'mkdir', autospec=True)
     def test_publish_image_external_ip_node_override(
-            self, mock_mkdir, mock_link, mock_shutil, mock_chmod):
+            self, mock_mkdir, mock_link, mock_shutil, mock_chmod,
+            mock_execute):
         self.config(use_swift=False, group='redfish')
         self.config(http_url='http://localhost',
                     external_http_url='http://non-local.host',
                     group='deploy')
         img_handler_obj = image_utils.ImageHandler(self.node.driver)
         self.node.driver_info["external_http_url"] = "http://node.override.url"
-
         override_url = self.node.driver_info.get("external_http_url")
-
         url = img_handler_obj.publish_image('file.iso', 'boot.iso',
                                             override_url)
-
         self.assertEqual(
             'http://node.override.url/redfish/boot.iso', url)
-
         mock_mkdir.assert_called_once_with('/httpboot/redfish', 0o755)
         mock_link.assert_called_once_with(
             'file.iso', '/httpboot/redfish/boot.iso')
         mock_chmod.assert_called_once_with('file.iso', 0o644)
+        mock_execute.assert_called_once_with(
+            '/usr/sbin/restorecon', '-i', '-R', 'v', '/httpboot/redfish')
 
     @mock.patch.object(os, 'chmod', autospec=True)
     @mock.patch.object(image_utils, 'shutil', autospec=True)
@@ -535,11 +558,14 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
 
             mock_unpublish.assert_called_once_with(mock.ANY, object_name)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_uefi(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             task.node.instance_info.update(deploy_boot_mode='uefi')
@@ -555,23 +581,26 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             object_name = 'boot-%s.iso' % task.node.uuid
 
             mock_publish_image.assert_called_once_with(
-                mock.ANY, mock.ANY, object_name)
+                mock.ANY, mock.ANY, object_name, None)
 
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href='http://bootloader/img',
-                kernel_params='nofb nomodeset vga=normal',
+                kernel_params='nofb vga=normal ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
 
             self.assertEqual(expected_url, url)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_default_boot_mode(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
         self.config(default_boot_mode='uefi', group='deploy')
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             image_utils._prepare_iso_image(
@@ -581,16 +610,54 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params='nofb nomodeset vga=normal',
+                kernel_params='nofb vga=normal ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
+    @mock.patch.object(image_utils.ImageHandler, 'publish_image',
+                       autospec=True)
+    @mock.patch.object(images, 'create_boot_iso', autospec=True)
+    def test__prepare_iso_image_with_node_external_http_url(
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        self.config(default_boot_mode='uefi', group='deploy')
+        mock_generate_uuid.return_value = '1-23-4'
+        with task_manager.acquire(self.context, self.node.uuid,
+                                  shared=True) as task:
+            override_url = 'https://node.external/'
+            task.node.driver_info.update(external_http_url=override_url)
+
+            expected_url = 'https://node.external/c.f?e=f'
+            mock_publish_image.return_value = expected_url
+
+            url = image_utils._prepare_iso_image(
+                task, 'http://kernel/img', 'http://ramdisk/img',
+                'http://bootloader/img', root_uuid=task.node.uuid)
+
+            object_name = 'boot-%s.iso' % task.node.uuid
+
+            mock_publish_image.assert_called_once_with(
+                mock.ANY, mock.ANY, object_name, override_url)
+
+            mock_create_boot_iso.assert_called_once_with(
+                mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
+                boot_mode='uefi', esp_image_href='http://bootloader/img',
+                kernel_params='nofb vga=normal ir_pub_id=1-23-4',
+                root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
+                inject_files=None, publisher_id='1-23-4')
+
+            self.assertEqual(expected_url, url)
+
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_bios(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
         self.config(default_boot_mode='bios', group='deploy')
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
 
@@ -605,22 +672,25 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             object_name = 'boot-%s.iso' % task.node.uuid
 
             mock_publish_image.assert_called_once_with(
-                mock.ANY, mock.ANY, object_name)
+                mock.ANY, mock.ANY, object_name, None)
 
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='bios', esp_image_href=None,
-                kernel_params='nofb nomodeset vga=normal',
+                kernel_params='nofb vga=normal ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
 
             self.assertEqual(expected_url, url)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -634,15 +704,19 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params=kernel_params,
+                kernel_params=f'{kernel_params} ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
+        self.assertEqual(1, mock_generate_uuid.call_count)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params_driver_info(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -656,15 +730,19 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params=kernel_params,
+                kernel_params=f'{kernel_params} ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
+        self.assertEqual(1, mock_generate_uuid.call_count)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params_defaults(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -679,16 +757,21 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params=f'nofb nomodeset vga=normal {kernel_params}',
+                kernel_params=(f'nofb vga=normal {kernel_params} '
+                               'ir_pub_id=1-23-4'),
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
+        self.assertEqual(1, mock_generate_uuid.call_count)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params_driver_info_bios(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
         self.config(default_boot_mode='bios', group='deploy')
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -702,16 +785,20 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='bios', esp_image_href=None,
-                kernel_params=kernel_params,
+                kernel_params=f'{kernel_params} ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None,
+                publisher_id='1-23-4')
+        self.assertEqual(1, mock_generate_uuid.call_count)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params_for_ramdisk_uefi(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -725,17 +812,21 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params="root=/dev/ram0 text " + kernel_params,
+                kernel_params=f'root=/dev/ram0 text {kernel_params}',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
                 inject_files=None)
+        mock_generate_uuid.assert_not_called()
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params_for_ramdisk_bios(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
         self.config(default_boot_mode='bios', group='deploy')
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -749,16 +840,20 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='bios', esp_image_href=None,
-                kernel_params="root=/dev/ram0 text " + kernel_params,
+                kernel_params=f'root=/dev/ram0 text {kernel_params}',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
                 inject_files=None)
+        mock_generate_uuid.assert_not_called()
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(deploy_utils, 'get_boot_option', lambda node: 'ramdisk')
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_kernel_params_for_ramdisk_cleaning(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -773,15 +868,19 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params=kernel_params,
+                kernel_params=f'{kernel_params} ir_pub_id=1-23-4',
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
+        self.assertEqual(1, mock_generate_uuid.call_count)
 
+    @mock.patch.object(uuidutils, 'generate_uuid', autospec=True)
     @mock.patch.object(image_utils.ImageHandler, 'publish_image',
                        autospec=True)
     @mock.patch.object(images, 'create_boot_iso', autospec=True)
     def test__prepare_iso_image_extra_params(
-            self, mock_create_boot_iso, mock_publish_image):
+            self, mock_create_boot_iso, mock_publish_image,
+            mock_generate_uuid):
+        mock_generate_uuid.return_value = '1-23-4'
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             kernel_params = 'network-config=base64-cloudinit-blob'
@@ -796,9 +895,12 @@ class RedfishImageUtilsTestCase(db_base.DbTestCase):
             mock_create_boot_iso.assert_called_once_with(
                 mock.ANY, mock.ANY, 'http://kernel/img', 'http://ramdisk/img',
                 boot_mode='uefi', esp_image_href=None,
-                kernel_params=kernel_params + ' foo=bar banana',
+                kernel_params=(f'{kernel_params} ir_pub_id=1-23-4 '
+                               'foo=bar banana'),
                 root_uuid='1be26c0b-03f2-4d2e-ae87-c02d7f33c123',
-                inject_files=None)
+                inject_files=None, publisher_id='1-23-4')
+        self.assertEqual(1, mock_generate_uuid.call_count)
+        self.assertEqual(1, mock_generate_uuid.call_count)
 
     def test__prepare_iso_image_bootable_iso(self):
         with task_manager.acquire(self.context, self.node.uuid,
